@@ -1,11 +1,11 @@
 package SVN::Notify;
 
-# $Id: Notify.pm 2724 2006-03-06 00:33:09Z theory $
+# $Id: Notify.pm 2768 2006-03-31 23:49:04Z theory $
 
 use strict;
 use constant WIN32  => $^O eq 'MSWin32';
 use constant PERL58 => $] > 5.007;
-$SVN::Notify::VERSION = '2.54';
+$SVN::Notify::VERSION = '2.55';
 
 =begin comment
 
@@ -207,6 +207,19 @@ The character set typically used on the repository for log messages, file
 names, and file contents. Used to specify the character set in the email
 Content-Type headers. Defaults to "UTF-8".
 
+=item io_layer
+
+  svnnotify --io-layer raw
+  svnnotify -o bytes
+
+The Perl IO layer to use for inputting and outputting data. See
+L<perlio|perlio> for details. Defaults to "encoding($charset)". If your
+repository uses different character encodings, C<charset> should be set to
+whatever is the most common character encoding, and C<io_layer> is best set to
+C<raw>. In that case, some characters might not look right in the commit
+messaage (because an email can manage only one character encoding at a time),
+but then C<svnnotify> won't get stuck inssuing a slew of warnings.
+
 =item language
 
   svnnotify --language fr
@@ -332,8 +345,10 @@ cause a message to bounce due to message size.
 Specify the subclass of SVN::Notify to be constructed and returned, and
 therefore to handle the notification. Of course you can just use a subclass
 directly, but this parameter is designed to make it easy to just use
-C<< SVN::Notify->new >> without worrying about loading subclasses, such as
-in F<svnnotify>.
+C<< SVN::Notify->new >> without worrying about loading subclasses, such as in
+F<svnnotify>. Be sure to read the documentation for your subclass of choice,
+as there may be additional parameters and existing parameters may behave
+differently.
 
 =item svnweb_url
 
@@ -466,6 +481,7 @@ sub new {
     $params{with_diff} ||= $params{attach_diff};
     $params{verbose}   ||= 0;
     $params{charset}   ||= 'UTF-8';
+    $params{io_layer}  ||= "encoding($params{charset})";
 
     # svnweb_url and viewcvs_url are mutually exlusive.
     if ($params{svnweb_url} && $params{svnweb_url} !~ /%s/) {
@@ -570,6 +586,7 @@ sub get_options {
         'svnlook|l=s'         => \$opts->{svnlook},
         'sendmail|s=s'        => \$opts->{sendmail},
         'charset|c=s'         => \$opts->{charset},
+        'io-layer|o=s'        => \$opts->{io_layer},
         'language|g=s'        => \$opts->{language},
         'with-diff|d'         => \$opts->{with_diff},
         'attach-diff|a'       => \$opts->{attach_diff},
@@ -718,8 +735,9 @@ sub prepare_recipients {
     $self->_dbpnt( "Preparing recipients list") if $self->{verbose};
     return $self unless $self->{to_regex_map} || $self->{subject_cx};
     my @to = $self->{to} ? ($self->{to}) : ();
-    my $regexen = delete $self->{to_regex_map};
+    my $regexen = $self->{to_regex_map};
     if ($regexen) {
+        $regexen = {%$regexen};
         $self->_dbpnt( "Compiling regex_map regular expressions")
           if $self->{verbose} > 1;
         for (values %$regexen) {
@@ -732,7 +750,7 @@ sub prepare_recipients {
         $regexen = {};
     }
 
-    my ($len, $cx);
+    my $cx;
     my $fh = $self->_pipe('-|', $self->{svnlook}, 'dirs-changed',
                           $self->{repos_path}, '-r', $self->{revision});
 
@@ -751,10 +769,12 @@ sub prepare_recipients {
         if ($self->{subject_cx}) {
             # XXX Do we need to set utf8 here?
             my $l = length;
-            ($len, $cx) = ($l, $_) unless defined $len && $len < $l;
-            $self->_dbpnt( qq{Context is "$cx"}) if $self->{verbose} > 1;
+            $cx ||= $_;
+            $cx =~ s{[/\\]?[^/\\]+$}{} until !$cx || /^$cx/;
         }
     }
+    $self->_dbpnt( qq{Context is "$cx"})
+        if $self->{subject_cx} && $self->{verbose} > 1;
     close $fh or warn "Child process exited: $?\n";
     $self->{to} = join ', ', @to;
     $self->{cx} = $cx;
@@ -1349,6 +1369,7 @@ __PACKAGE__->_accessors(qw(
     svnlook
     sendmail
     charset
+    io_layer
     language
     with_diff
     attach_diff
@@ -1462,6 +1483,13 @@ Gets or sets the value of the C<sendmail> attribute.
 
 Gets or sets the value of the C<charset> attribute.
 
+=head3 io_layer
+
+  my $io_layer = $notifier->io_layer;
+  $notifier = $notifier->io_layer($io_layer);
+
+Gets or sets the value of the C<io_layer> attribute.
+
 =head3 language
 
   my $language = $notifier->language;
@@ -1629,16 +1657,16 @@ sub _pipe {
     # Safer version of backtick (see perlipc(1)).
     # XXX Use Win32::Process on Win32? This doesn't seem to work as-is on Win32.
     local *PIPE;
-    my $pid = open(PIPE, $mode);
+    my $pid = open PIPE, $mode;
     die "Cannot fork: $!\n" unless defined $pid;
 
     if ($pid) {
         # Parent process. Set the encoing layer and return the file handle.
-        binmode(PIPE, ':encoding(' . $self->charset . ')') if PERL58;
+        binmode PIPE, ":$self->{io_layer}" if PERL58;
         return *PIPE;
     } else {
         # Child process. Execute the commands.
-        exec(@_) or die "Cannot exec $_[0]: $!\n";
+        exec @_ or die "Cannot exec $_[0]: $!\n";
         # Not reached.
     }
 }
@@ -1673,8 +1701,14 @@ __END__
 
 =item L<SVN::Notify::HTML|SVN::Notify::HTML>
 
+=item L<SVN::Notify::HTML::ColorDiff|SVN::Notify::HTML::ColorDiff>
+
 Subclasses SVN::Notify.
 
+=item L<https://sourceforge.net/docs/E09#svn_notify>
+
+SourceForge.net support for SVN::Notify.
+
 =back
 
 =head1 Bugs
